#!/usr/bin/env python3
'''Do some counting on the NEWS data.'''

from collections import defaultdict
from glob import glob
from optparse import OptionParser
from os import path
from string import punctuation
from sys import argv, stdin
from xml.etree import ElementTree

__author__ = 'Aditya Bhargava'
__license__ = 'FreeBSD'
__version__ = '1.0.0'
__email__ = 'aditya@cs.toronto.edu'


def errordie(msg):
    print(argv[0], ': error: ', msg, sep='')
    exit()

parser = OptionParser(usage='usage: %prog [options args] [DIR] < names.asc')
parser.add_option('-c', '--countwords', action='store_true',
                  dest='countwords', default=False,
                  help='split NEWS strings and add individual word pairs')
parser.add_option('-n', '--nospacemismatch', action='store_true',
                  dest='nospacemm', default=False,
                  help="ignore NEWS data that has a word count mismatch")

(options, args) = parser.parse_args()
if options.countwords:
    options.nospacemm = True

TLDIR = ''
if len(args) == 1:
    TLDIR = args[0]
elif len(args) > 1:
    errordie('too many arguments')

transliterations = defaultdict(set)
langs = set()
for dir in sorted(glob(path.join(TLDIR, 'En??/'))):
    lang = dir[-3:-1]
    langs.add(lang)
    for datafile in sorted(glob(path.join(dir, '*.xml'))):
        tree = ElementTree.parse(datafile)
        corpus = tree.getroot()
        for tlset in corpus:
            src = tlset[0].text.strip().lower()
            for name in tlset:
                cln = name.text.strip().lower()
                clns, srcs = cln.split(' '), src.split(' ')
                if 'Target' in name.tag and ((not options.nospacemm) or
                                             len(clns) == len(srcs)):
                    if options.countwords and len(srcs) > 1:
                        for src in srcs:
                            transliterations[lang].add(src)
                    else:
                        transliterations[lang].add(src)

for lang in sorted(langs):
    print('{}: {}'.format(lang, len(transliterations[lang])))
