#!/usr/bin/env python3

from collections import defaultdict
from optparse import OptionParser
from subprocess import getoutput
from sys import argv, stdin, stderr

__author__ = 'Aditya Bhargava'
__license__ = 'FreeBSD'
__version__ = '1.0.0'
__email__ = 'aditya@cs.toronto.edu'


def errordie(msg, exitcode):
    print(argv[0], ': error: ', msg, sep='', file=stderr)
    exit(exitcode)

parser = OptionParser(usage='usage: %prog [options] REF ORIG BASENAME < RR')

(options, args) = parser.parse_args()

if len(args) == 0:
    errordie('no reference file specified', 2)
elif len(args) == 1:
    errordie('no original output file specified', 2)
elif len(args) == 2:
    errordie('no basename specified', 2)
elif len(args) > 3:
    errordie('too many options specified', 2)

reffile, origfile, basename = args

refdata = defaultdict(list)
with open(reffile) as inp:
    for line in inp:
        k, v = line.strip().split('\t')
        refdata[k].append(v)

origdata = {}
with open(origfile) as inp:
    for line in inp:
        k, v = line.strip().split('\t')
        if k not in origdata:
            origdata[k] = v

data = defaultdict(list)
for line in stdin.readlines():
    k, v = line.strip().split('\t')[0:2]
    if v in refdata[k]:
        myline = '* ' + line
    elif v == origdata[k]:
        myline = 'X ' + line
    else:
        myline = '  ' + line
    data[k].append((v, myline))

good, bad, better, worse = [], [], [], []
for key in data:
    tst = data[key][0][0]
    orig = origdata[key]
    ref = refdata[key]

    if tst in ref:
        if orig in ref:
            good.extend(data[key])
        else:
            better.extend(data[key])
    else:
        if orig in ref:
            worse.extend(data[key])
        else:
            bad.extend(data[key])

goodfile = basename + '.good'
badfile = basename + '.bad'
worsefile = basename + '.worse'
betterfile = basename + '.better'

with open(goodfile, 'w') as outp:
    for ph, line in good:
        outp.write(line)
with open(badfile, 'w') as outp:
    for ph, line in bad:
        outp.write(line)
with open(worsefile, 'w') as outp:
    for ph, line in worse:
        outp.write(line)
with open(betterfile, 'w') as outp:
    for ph, line in better:
        outp.write(line)
