#!/usr/bin/env python3

from math import ceil, log
from sys import stdin

__author__ = 'Aditya Bhargava'
__license__ = 'FreeBSD'
__version__ = '1.0.0'
__email__ = 'aditya@cs.toronto.edu'

NBEST = 10


def permute(outlists):
    permutations = [(letter + '|', logprob) for letter, logprob in
                    outlists[0][:NBEST + 1]]

    for outlist in outlists[1:]:
        newpermutations = sorted([(partial + letter + '|', lpsum + logprob) for
                                  partial, lpsum in permutations for letter,
                                  logprob in outlist], reverse=True, key=lambda
                                 t: t[1])[:NBEST + 1]
        permutations = newpermutations
        newpermutations = []

    permutations = [(permutation, logprob) for permutation, logprob in
                    permutations if
                    permutation.replace('|', '').replace('_', '')][:NBEST]
    return permutations

buffer, letters, words = [], [], []
for line in stdin.readlines():
    stuff = line.split()[1:]
    buffer.append(stuff[4])

    outs = []
    for i in range(9, len(stuff) - 1, 2):
        letter = stuff[i].replace('-', '').replace('_epsilon_', '_')
        prob = float(stuff[i + 1])
        if prob > 0:
            outs.append((letter, log(prob)))
    letters.append(outs)

    if stuff[5:9] == ['#', '0', '0', '0']:
        word = '|'.join(buffer) + '|'
        if word not in words:
            words.append(word)
            outputs = permute(letters)
            if len(outputs) < NBEST:
                outputs *= ceil(NBEST / len(outputs))
                outputs = outputs[:NBEST]
            for output in outputs:
                print(word, output[0], output[1], sep='\t')
        buffer, letters = [], []
