#!/usr/bin/env python3

from collections import defaultdict
from glob import glob
from optparse import OptionParser
from os import path
from string import punctuation
from sys import argv, stdin
from xml.etree import ElementTree

__author__ = 'Aditya Bhargava'
__license__ = 'FreeBSD'
__version__ = '1.0.0'
__email__ = 'aditya@cs.toronto.edu'


def errordie(msg):
    print(argv[0], ': error: ', msg, sep = '')
    exit()

parser = OptionParser(usage = 'usage: %prog [options args] [DIR] < names.asc')
parser.add_option('-c', '--countwords', action = 'store_true', dest =
                  'countwords', default = False, help =
                  'split NEWS strings and add individual word pairs')
parser.add_option('-n', '--nospacemismatch', action = 'store_true', dest =
                  'nospacemm', default = False, help =
                  "ignore NEWS data that has a word count mismatch")

(options, args) = parser.parse_args()
if options.countwords:
    options.nospacemm = True

TLDIR = ''
if len(args) == 1:
    TLDIR = args[0]
elif len(args) > 1:
    errordie('too many arguments')

pronunciations = defaultdict(set)
for line in stdin.readlines():
    k, v = line.strip().split('\t')
    pronunciations[k].add(v)

transliterations = defaultdict(lambda : defaultdict(set))
langs = set()
for dir in sorted(glob(path.join(TLDIR, 'En??/'))):
    lang = dir[-3 : -1]
    langs.add(lang)
    for datafile in sorted(glob(path.join(dir, '*.xml'))):
        tree = ElementTree.parse(datafile)
        corpus = tree.getroot()
        for tlset in corpus:
            src = tlset[0].text.strip().lower()
            for name in tlset:
                cln = name.text.strip().lower()
                clns, srcs = cln.split(' '), src.split(' ')
                if 'Target' in name.tag and ((not options.nospacemm) or
                                             len(clns) == len(srcs)):
                    transliterations[src][lang].add(cln)
                    if options.countwords and len(srcs) > 1:
                        for i in range(len(srcs)):
                            srcw = ''.join(c for c in srcs[i] if c not in
                                           punctuation)
                            dstw = ''.join(c for c in clns[i] if c not in
                                           punctuation)
                            transliterations[srcw][lang].add(dstw)

cset = set(pronunciations.keys())
nset = set(transliterations.keys())

common = cset & nset
for lang in sorted(langs):
    with open('En{}Ph'.format(lang), 'w') as outp:
        for name in sorted(common):
            for dst in sorted(transliterations[name][lang]):
                for ph in sorted(pronunciations[name]):
                    outp.write('\t'.join((name, dst, ph)) + '\n')
