#!/usr/bin/env python3

import optparse
from sys import stdin

__author__ = 'Aditya Bhargava'
__license__ = 'FreeBSD'
__version__ = '1.0.0'
__email__ = 'aditya@cs.toronto.edu'


parser = optparse.OptionParser(usage = 'usage: %prog [options args] < DATA')
parser.add_option('-i', '--include-only', dest = 'include', type = 'string',
                  default = "", help = 'inclusion-only file')
parser.add_option('-x', '--exclude', dest = 'exclude', type = 'string',
                  default = "", help = 'exclusion file')
parser.add_option('-s', '--no-spacify', dest = 'spacify',
                  action = 'store_false', default = True,
                  help = "don't insert spaces between output characters")
parser.add_option('-r', '--remove-spaces', dest = 'removespaces',
                  action = 'store_true', default = False,
                  help = 'remove spaces from input (useful for m2m files)')

(options, args) = parser.parse_args()

exclude = set()
if options.exclude:
    with open(options.exclude) as inp:
        for line in inp:
            exclude.add(line.strip().split('\t')[0])

include = set()
if options.include:
    with open(options.include) as inp:
        for line in inp:
            include.add(line.strip().split('\t')[0])

data = []
for line in stdin.readlines():
    stuff = line.strip().split('\t')
    check = stuff[0]
    if options.removespaces:
        check = stuff[0].replace(' ', '')
    if not options.exclude or check not in exclude:
        if not options.include or check in include:
            if options.spacify:
                print(' '.join(check), ' '.join(stuff[1]), ' '.join(stuff[2]),
                      sep = '\t')
            else:
                print(line, end = '')
