#!/usr/bin/env python3

from collections import defaultdict
from itertools import product
from optparse import OptionParser
from sys import argv, stdin, stderr


def errordie(msg, exitcode):
    print(argv[0], ': error: ', msg, sep = '')
    exit(exitcode)

parser = OptionParser(usage = 'usage: %prog INEXOUT < SYSOUT')

(options, args) = parser.parse_args()

if len(args) == 0:
    errordie('no InExOut file(s) specified', 2)

enhiphfile = args[0]

allfodata = []
for enfophfile in args:
    enfodata = defaultdict(list)
    with open(enfophfile) as inp:
        for line in inp:
            en, fo = line.strip().split('\t')[:2]
            encln = en.replace(' ', '')
            if fo not in enfodata[encln]:
                enfodata[encln].append(fo)
    allfodata.append(enfodata)

data = defaultdict(list)
for line in stdin.readlines():
    stuff = line.strip().split('\t')
    if len(stuff) == 3:
        k, v, s = stuff
    elif len(stuff) == 4:
        k, f1, f2, f3 = line.strip().split('\t')
        try:
            r, s, v = int(f1), f2, f3
        except ValueError:
            v, r, s = f1, int(f2), f3
    data[''.join(c for c in k if c not in '|_')].append((k, v, s))

for encln in data:
    for en, ph, sc in data[encln]:
        line = '\t'.join((en, ph, sc)) + '\t'
        phcln = ''.join(c for c in ph if c not in '|_')
        insome = False
        for enfodata in allfodata:
            if enfodata[encln]:
                for fo in enfodata[encln]:
                    insome = True
                    if ' ' in fo:
                        line += fo
                    else:
                        line += ' '.join(fo)
                    break
            else:
                line += 'NULL'
            line += '\t'

        if insome:
            print(line + ' '.join(c for c in ph if c not in '|_'))
